@@ -2,14 +2,55 @@ from __future__ import annotations
 
 import abc
 import argparse
+from dataclasses import dataclass
 import os.path
 import traceback
-from typing import Any, Callable, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Optional, TYPE_CHECKING
 
 import e3.collection.dag
 import e3.env
 
-from e3.testsuite.result import TestResult
+from e3.testsuite.report.gaia import (
+    GAIAResultFiles,
+    dump_result_logs_if_needed,
+)
+from e3.testsuite.result import TestResult, TestResultSummary
+from e3.testsuite.utils import CleanupMode
+
+if TYPE_CHECKING:
+    from e3.testsuite.fragment import FragmentCallback
+
+
+@dataclass(frozen=True)
+class ResultQueueItem:
+    """Information to integrate a test result in a testsuite report.
+
+    Test drivers create test results. They travel from there to the testsuite
+    final report through various queues. This class gathers all the information
+    needed for the various stages of this pipeline.
+    """
+
+    result: TestResultSummary
+    """Summary for this test result."""
+
+    filename: str
+    """Name of the file that contains test result data."""
+
+    traceback: List[str]
+    """Stack trace for this result's push time.
+
+    This stack trace corresponds to the code that led the TestResult instance
+    to be included in the testsuite report.
+    """
+
+    gaia_results: Optional[GAIAResultFiles]
+    """GAIA files for this result.
+
+    This is None if no GAIA report is requested.
+    """
+
+
+ResultQueue = List[ResultQueueItem]
 
 
 class TestDriver(object, metaclass=abc.ABCMeta):
@@ -18,7 +59,12 @@ class TestDriver(object, metaclass=abc.ABCMeta):
     All drivers declared in a testsuite should inherit from this class
     """
 
-    def __init__(self, env: e3.env.BaseEnv, test_env: Dict[str, Any]) -> None:
+    Fore: Any
+    Style: Any
+
+    test_env_filename: str
+
+    def __init__(self, env: e3.env.Env, test_env: Dict[str, Any]) -> None:
         """Initialize a TestDriver instance.
 
         :param env: The testsuite environment. This mirrors the
@@ -35,7 +81,7 @@ class TestDriver(object, metaclass=abc.ABCMeta):
               that this test driver is free to create (if needed) in order to
               run the testcase.
         """
-        self.env: e3.env.BaseEnv = env
+        self.env: e3.env.Env = env
         assert isinstance(env.options, argparse.Namespace)
         self.testsuite_options: argparse.Namespace = env.options
 
@@ -47,11 +93,11 @@ class TestDriver(object, metaclass=abc.ABCMeta):
             name=self.test_name, env=self.test_env
         )
 
-        # Queue used to push result to the testsuite. Each queue item is a
-        # couple that contains the TestResult instance and a string traceback
-        # corresponding to the chain of call that pushed that result. This
-        # traceback is useful to debug test drivers that push twice results.
-        self.result_queue: List[Tuple[TestResult, List[str]]] = []
+        self.result_queue: ResultQueue = []
+        """
+        Queue of test results that this driver plans to integrate to the
+        testsuite report.
+        """
 
     def push_result(self, result: Optional[TestResult] = None) -> None:
         """Push a result to the testsuite.
@@ -63,20 +109,33 @@ class TestDriver(object, metaclass=abc.ABCMeta):
         """
         if result is None:
             result = self.result
-        self.result_queue.append((result, traceback.format_stack()))
+
+        # Write the result as a new file in the output directory and let the
+        # testsuite main know about this result.
+        output_dir = self.env.output_dir
+        assert isinstance(output_dir, str)
+
+        self.result_queue.append(
+            ResultQueueItem(
+                result.summary,
+                result.save(self.env.output_dir),
+                traceback.format_stack(),
+                dump_result_logs_if_needed(self.env, result, output_dir),
+            )
+        )
 
     def add_fragment(
         self,
         dag: e3.collection.dag.DAG,
         name: str,
-        fun: Optional[Callable[[Dict[str, Any], int], None]] = None,
-        after: Optional[List[str]] = None
+        fun: Optional[FragmentCallback] = None,
+        after: Optional[List[str]] = None,
     ) -> None:
         """Add a test fragment.
 
         This function is a helper to define test workflows that do not
-        introduce dependencies to other tests. For more complex operation
-        use directly add_vertex method from the dag. See add_test method
+        introduce dependencies to other tests. For more complex operation use
+        directly add_vertex method from the dag. See add_test method.
 
         :param dag: DAG containing test fragments.
         :param name: Name of the fragment.
@@ -87,26 +146,39 @@ class TestDriver(object, metaclass=abc.ABCMeta):
         :param after: List of fragment names that should be executed before
             this one.
         """
+        from e3.testsuite.fragment import FragmentData
+
+        callback_by_name = False
+
         if after is not None:
             after = [self.test_name + "." + k for k in after]
 
         if fun is None:
             fun = getattr(self, name)
+            callback_by_name = True
+
+        fragment = FragmentData(
+            uid=f"{self.test_name}.{name}",
+            driver=self,
+            name=name,
+            callback=fun,
+            callback_by_name=callback_by_name,
+        )
 
         dag.update_vertex(
-            vertex_id=self.test_name + "." + name,
-            data=(self, fun),
+            vertex_id=fragment.uid,
+            data=fragment,
             predecessors=after,
-            enable_checks=False
+            enable_checks=False,
         )
 
     def working_dir(self, *args: str) -> str:
         """Build a filename in the test working directory."""
-        return os.path.join(self.test_env['working_dir'], *args)
+        return os.path.join(self.test_env["working_dir"], *args)
 
     def test_dir(self, *args: str) -> str:
         """Build a filename in the testcase directory."""
-        return os.path.join(self.test_env['test_dir'], *args)
+        return os.path.join(self.test_env["test_dir"], *args)
 
     @abc.abstractmethod
     def add_test(self, dag: e3.collection.dag.DAG) -> None:
@@ -127,7 +199,7 @@ class TestDriver(object, metaclass=abc.ABCMeta):
         whole testsuite are removed incrementally. This is necessary to avoid
         creating huge temporary directories when executing big testsuites.
         """
-        return self.testsuite_options.enable_cleanup
+        return self.env.cleanup_mode != CleanupMode.NONE
 
 
 class BasicTestDriver(TestDriver, metaclass=abc.ABCMeta):
