@@ -6,18 +6,7 @@ from e3.diff import diff
 from e3.os.fs import unixpath
 from e3.testsuite.result import FailureReason, Log, binary_repr, truncated
 from e3.testsuite.driver.classic import ClassicTestDriver, TestAbortWithError
-
-
-def indent(text: str, prefix: str = "  ") -> str:
-    """Prepend ``prefix`` to every line in ``text``.
-
-    :param text: Text to transform.
-    :param prefix: String to prepend.
-    """
-    # Use .split() rather than .splitlines() because we need to preserve the
-    # last line if is empty. "a\n".splitlines() returns ["a"], so we must avoid
-    # it.
-    return "\n".join((prefix + line) for line in text.split("\n"))
+from e3.testsuite.utils import indent
 
 
 class OutputRefiner(Generic[AnyStr]):
@@ -197,7 +186,7 @@ class DiffTestDriver(ClassicTestDriver):
                 "cannot read baseline file ({}: {})".format(
                     type(exc).__name__, exc
                 )
-            )
+            ) from exc
         return (filename, baseline, is_regexp)
 
     @property
@@ -237,6 +226,11 @@ class DiffTestDriver(ClassicTestDriver):
         """
         return False
 
+    @property
+    def diff_context_size(self) -> int:
+        """Positive number of context lines to include in diff computations."""
+        return self.test_env.get("diff_context_size", 1)
+
     def set_up(self) -> None:
         super().set_up()
 
@@ -250,6 +244,7 @@ class DiffTestDriver(ClassicTestDriver):
         actual: AnyStr,
         failure_message: str = "unexpected output",
         ignore_white_chars: Optional[bool] = None,
+        context_size: Optional[int] = None,
         truncate_logs_threshold: Optional[int] = None,
     ) -> List[str]:
         """Compute the diff between expected and actual outputs.
@@ -265,6 +260,8 @@ class DiffTestDriver(ClassicTestDriver):
         :param ignore_white_chars: Whether to ignore whitespaces during the
             diff computation. If left to None, use
             ``self.diff_ignore_white_chars``.
+        :param context_size: Positive number of context lines to include in
+            diff computations. If left to None, use ``self.diff_context_size``.
         :param truncate_logs_threshold: Threshold to truncate the diff message
             in ``self.result.log``. See ``e3.testsuite.result.truncated``'s
             ``line_count`` argument. If left to None, use the testsuite's
@@ -273,6 +270,9 @@ class DiffTestDriver(ClassicTestDriver):
         if ignore_white_chars is None:
             ignore_white_chars = self.diff_ignore_white_chars
 
+        if context_size is None:
+            context_size = self.diff_context_size
+
         if truncate_logs_threshold is None:
             truncate_logs_threshold = self.testsuite_options.truncate_logs
 
@@ -308,7 +308,10 @@ class DiffTestDriver(ClassicTestDriver):
         # include the diff in the test log and return the given failure
         # message.
         d = diff(
-            expected_lines, actual_lines, ignore_white_chars=ignore_white_chars
+            expected_lines,
+            actual_lines,
+            ignore_white_chars=ignore_white_chars,
+            context=context_size,
         )
         if not d:
             return []
@@ -408,7 +411,7 @@ class DiffTestDriver(ClassicTestDriver):
         )
         refined_actual = refiners.refine(actual)
 
-        match = regexp.match(refined_actual)
+        match = regexp.fullmatch(refined_actual)
         if match:
             return []
 
