@@ -5,10 +5,12 @@ This checks the behavior of the Testsuite, TestDriver and BasicTestDriver
 classes.
 """
 
+import gc
 import os
 import re
 import time
 import warnings
+import weakref
 
 from e3.fs import mkdir, mv
 from e3.testsuite import TestAbort as E3TestAbort, Testsuite as Suite
@@ -16,6 +18,7 @@ from e3.testsuite.driver import (
     TestDriver as Driver,
     BasicTestDriver as BasicDriver,
 )
+from e3.testsuite.driver.diff import DiffTestDriver as DiffDriver
 from e3.testsuite.report.index import ReportIndex
 from e3.testsuite.result import (
     TestResult as Result,
@@ -32,7 +35,7 @@ from .utils import (
     extract_results,
     run_testsuite,
     run_testsuite_status,
-    testsuite_logs,
+    suite_logs,
 )
 
 
@@ -59,7 +62,8 @@ class TestBasic:
             return "default"
 
     def test(self):
-        # Run the testsuite and check both the in-memory report and the on-disk one
+        # Run the testsuite and check both the in-memory report and the on-disk
+        # one.
         result = {"test1": Status.PASS, "test2": Status.PASS}
         suite = run_testsuite(self.Mysuite)
         assert extract_results(suite) == result
@@ -123,7 +127,7 @@ class TestInvalidFilterPattern:
             message.startswith(
                 "Test pattern is not a valid regexp, try to match it as-is: "
             )
-            for message in testsuite_logs(caplog)
+            for message in suite_logs(caplog)
         )
 
 
@@ -176,7 +180,7 @@ class TestNoTestcase:
 
     def test(self, caplog):
         suite = run_testsuite(self.Mysuite)
-        logs = testsuite_logs(caplog)
+        logs = suite_logs(caplog)
         assert extract_results(suite) == {}
         assert any("<no test result>" in message for message in logs)
 
@@ -308,7 +312,7 @@ class TestNotExistingTempDir:
         # Check that the testsuite aborted and that we have the expected error
         # message.
         run_testsuite(self.Mysuite, ["--temp-dir=tmp"], expect_failure=True)
-        logs = testsuite_logs(caplog)
+        logs = suite_logs(caplog)
         assert any(
             re.search("temp dir '.*tmp' does not exist", entry)
             for entry in logs
@@ -561,7 +565,7 @@ class TestShowErrorOutput:
         run_testsuite(
             self.Mysuite, ["--show-error-output"], expect_failure=True
         )
-        logs = testsuite_logs(caplog)
+        logs = suite_logs(caplog)
         assert any("Work is being done" in message for message in logs)
 
 
@@ -1118,3 +1122,162 @@ def test_results_relocation(tmp_path):
     new_index = ReportIndex.read(new_dir)
     assert new_index.entries["foo"].load().status == Status.PASS
     assert new_index.entries["bar"].load().status == Status.FAIL
+
+
+def test_index_save_and_add(tmp_path):
+    """Check that ReportIndex.save_and_add_result works as expected."""
+    # Write a report with two results
+    index = ReportIndex(str(tmp_path))
+    index.save_and_add_result(Result("foo", status=Status.PASS))
+    index.save_and_add_result(Result("bar", status=Status.FAIL))
+    index.write()
+
+    # Read it and make sure the results can be read
+    index = ReportIndex.read(str(tmp_path))
+    assert index.entries["foo"].load().status == Status.PASS
+    assert index.entries["bar"].load().status == Status.FAIL
+
+
+class TestMemleak:
+    """Check the absence of memory leaks in a testsuite run."""
+
+    class MySuite(MultiSchedulingSuite):
+        def adjust_dag_dependencies(self, dag):
+            # Create our mapping to test fragments for convenience below
+            self.test_fragments = {}
+            for fg in dag.vertex_data.values():
+                if fg.matches(TestMemleak.MyDriver, "run_wrapper"):
+                    self.test_fragments[fg.driver.test_name] = fg
+            t1 = self.test_fragments["t1"]
+            t2 = self.test_fragments["t2"]
+
+            # Create weakrefs to critical objects so that we can check later,
+            # after a full GC collection, that no references to them are left.
+            self.test_wr = weakref.WeakValueDictionary()
+
+            # Make sure that t2 is executed after t1
+            dag.update_vertex(vertex_id=t2.uid, predecessors=[t1.uid])
+
+            # Flags to make sure that conditional code in collect_result were
+            # executed.
+            self.t1_result_collected = False
+            self.test_succeeded = False
+
+        def collect_result(self, fragment):
+            # Before the actual "collect_result" method has a chance to remove
+            # the last references to various data that we want to check, create
+            # weak references to them.
+            if fragment.uid is self.test_fragments["t1"].uid:
+                driver = fragment.driver
+                result = driver.result
+
+                for key, value in [
+                    ("t1_driver", driver),
+                    ("t1_result_log", result.log),
+                    # The "output" attribute is set during the test execution,
+                    # so in multiprocessing mode, it is never set for the
+                    # TestDriver instance in the testsuite "master" process.
+                    ("t1_output", getattr(driver, "output", None)),
+                    ("t1_result_out", result.out),
+                    ("t1_result_expected", result.expected),
+                ]:
+                    if value is not None:
+                        self.test_wr[key] = value
+                self.t1_result_collected = True
+
+            super().collect_result(fragment)
+
+            # By the time we collect results for t2, all the refs for t1 should
+            # be gone (they used to leak until the testsuite itself got
+            # collected). Trigger a GC collection and check the absence of
+            # weakref, which proves the absence of refs left to the critical
+            # objects.
+            if fragment.uid == self.test_fragments["t2"].uid:
+                gc.collect()
+                assert dict(self.test_wr) == {}
+                self.test_succeeded = True
+
+    class MyDriver(DiffDriver):
+        @property
+        def baseline(self):
+            return (None, "", False)
+
+        def run(self):
+            self.result.log += "This is some log"
+            self.output += "This is some output"
+
+    def run_check(self, multiprocessing):
+        suite = run_testsuite(
+            create_testsuite(["t1", "t2"], self.MyDriver, self.MySuite),
+            ["--failure-exit-code=0"],
+            multiprocessing=multiprocessing,
+        )
+        assert extract_results(suite) == {"t1": Status.FAIL, "t2": Status.FAIL}
+        assert suite.t1_result_collected
+        assert suite.test_succeeded
+
+    def test_multithread(self):
+        self.run_check(multiprocessing=False)
+
+    def test_multiprocess(self):
+        self.run_check(multiprocessing=True)
+
+
+class TestSkipPassed:
+    """Check that --skip-passed works as expected."""
+
+    class MyDriver(BasicDriver):
+        def run(self, prev, slot):
+            pass
+
+        def analyze(self, prev, slot):
+            _, status_name = self.test_name.split("-")
+            status = Status[status_name.upper()]
+            self.result.set_status(status)
+            self.push_result()
+
+    def test(self):
+        # Run the testsuite a first time. All tests should be scheduled and
+        # thus be present in the report.
+        tests_1 = [
+            "t1-pass",
+            "t1-fail",
+            "t1-xfail",
+            "t1-xpass",
+            "t1-verify",
+            "t1-skip",
+            "t1-not_applicable",
+            "t1-error",
+        ]
+        suite = run_testsuite(
+            create_testsuite(tests_1, self.MyDriver),
+            ["--skip-passed"],
+            expect_failure=True,
+        )
+        assert extract_results(suite) == {
+            "t1-pass": Status.PASS,
+            "t1-fail": Status.FAIL,
+            "t1-xfail": Status.XFAIL,
+            "t1-xpass": Status.XPASS,
+            "t1-verify": Status.VERIFY,
+            "t1-skip": Status.SKIP,
+            "t1-not_applicable": Status.NOT_APPLICABLE,
+            "t1-error": Status.ERROR,
+        }
+
+        # Run the testsuite a second time. All tests but PASS, XFAIL and XPASS
+        # ones should run, and the new one should run, too.
+        tests_2 = tests_1 + ["t2-pass"]
+        suite = run_testsuite(
+            create_testsuite(tests_2, self.MyDriver),
+            ["--skip-passed"],
+            expect_failure=True,
+        )
+        assert extract_results(suite) == {
+            "t1-fail": Status.FAIL,
+            "t1-verify": Status.VERIFY,
+            "t1-skip": Status.SKIP,
+            "t1-not_applicable": Status.NOT_APPLICABLE,
+            "t1-error": Status.ERROR,
+            "t2-pass": Status.PASS,
+        }
