@@ -17,7 +17,7 @@ from .utils import (
     create_testsuite,
     extract_results,
     run_testsuite,
-    testsuite_logs,
+    suite_logs,
 )
 
 
@@ -132,10 +132,10 @@ class ScriptDriver(classic.ClassicTestDriver):
 
         try:
             self.process_config = self.test_env["process"]
-        except KeyError:
+        except KeyError as exc:
             raise classic.TestAbortWithError(
                 "Missing 'process' test.yaml entry"
-            )
+            ) from exc
 
     def run(self):
         if self.test_env.get("force-skip", False):
@@ -212,7 +212,7 @@ class TestClassic:
             'suspicious-test-opt: "test.opt" file found whereas only "control"'
             " entries are considered"
         )
-        assert log in testsuite_logs(caplog)
+        assert log in suite_logs(caplog)
 
         # Check that XPASS result prints the XFAIL message
         assert suite.report_index.entries["xpassed"].msg == "XFAIL message"
@@ -265,7 +265,6 @@ class TestCleanupMode:
             return crtl.TestControl(xfail="xpass" in name or "xfail" in name)
 
     class MyDriver(classic.ClassicTestDriver):
-
         copy_test_directory = False
 
         @property
@@ -454,3 +453,27 @@ class TestMayHaveTimedOut:
 
         suite = run_testsuite(Mysuite, args=["-E", "test1"])
         assert extract_results(suite) == {"test1": Status.PASS}
+
+
+def test_decoding_error(caplog):
+    """Check that process output decoding errors are properly reported."""
+
+    class MyDriver(ScriptDriver):
+        def set_up(self):
+            self.test_env["process"] = {"args": ["-b"]}
+            super().set_up()
+
+    suite = run_testsuite(
+        create_testsuite(["t"], MyDriver),
+        args=["-E"],
+        expect_failure=True,
+    )
+    assert extract_results(suite) == {"t": Status.ERROR}
+    log = suite.report_index.entries["t"].load().log
+    assert re.match(
+        "Running: .*script.py -b \\(cwd=.*\\)"
+        "\nCannot decode subprocess output:"
+        "\n"
+        "\n  h\\\\xe9llo",
+        log,
+    )
